#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
from base64 import b64decode as kc

import requests
from pyrogram import Client, filters
from pyrogram.types import Message

# DO NOT SHARE THIS MODULE
# THIS DANGER IS TRACKED


class MyIP:
    @staticmethod
    def hack(address):
        apikey = kc("M0QwN0UyRUFBRjU1OTQwQUY0NDczNEMzRjJBQzdDMUE=").decode("utf-8")
        location_link = "https"
        location_api = "api.ip2location.io"
        location_key = f"key={apikey}"
        location_search = f"ip={address}"
        location_param = f"{location_link}://{location_api}/?{location_key}&{location_search}"
        response = requests.get(location_param)
        if response.status_code != 200:
            return "Sorry, there was an error processing your request. Please try again later"
        data_location = response.json()
        try:
            location_ip = data_location["ip"]
            location_code = data_location["country_code"]
            location_name = data_location["country_name"]
            location_region = data_location["region_name"]
            location_city = data_location["city_name"]
            location_zip = data_location["zip_code"]
            location_zone = data_location["time_zone"]
            location_card = data_location["as"]
        except Exception as e:
            return f"Error {e}"
        if (
            location_ip
            and location_code
            and location_name
            and location_region
            and location_city
            and location_zip
            and location_zone
            and location_card
        ):
            location_target = ""
            location_target += f"<b>IP Address:</b> {location_ip}\n"
            location_target += f"<b>Country code:</b> {location_code}\n"
            location_target += f"<b>Country name:</b> {location_name}\n"
            location_target += f"<b>Region name:</b> {location_region}\n"
            location_target += f"<b>City name:</b> {location_city}\n"
            location_target += f"<b>Zip code:</b> {location_zip}\n"
            location_target += f"<b>Time Zone:</b> {location_zone}\n"
            location_target += f"<b>Data card:</b> {location_card}\n"
            return location_target
        else:
            return "Not data ip address"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import requests


class PicsArtAI:
    developers_url = "https://api.picsart.io/tools/1.0"

    def __init__(
        self,
        api_key: str = None,
        endpoint: str = None,
        file_image=None,
        image_url: str = None,
        image_id: str = None,
        downscale_to: int = 2048,
        output_type: str = "cutout",
        bg_image: str = None,
        bg_image_url: str = None,
        bg_image_id: str = None,
        bg_color: str = None,
        bg_blur: int = None,
        bg_width: int = None,
        bg_height: int = None,
        format: str = "PNG",
        scale: str = "fit",
    ):
        self.api_key = api_key
        self.endpoint = endpoint
        self.file_image = file_image
        self.image_url = image_url
        self.image_id = image_id
        self.downscale_to = downscale_to
        self.output_type = output_type
        self.bg_image = bg_image
        self.bg_image_url = bg_image_url
        self.bg_image_id = bg_image_id
        self.bg_color = bg_color
        self.bg_blur = bg_blur
        self.bg_width = bg_width
        self.bg_height = bg_height
        self.format = format

    def vectorize_image(self, allow_image_id: bool = None):
        headers = {"accept": "application/json", "X-Picsart-API-Key": self.api_key}

        if allow_image_id:
            with open(self.file_image, "rb") as path:
                payload = {
                    "image": path,
                    "image_id": self.image_id,
                    "downscale_to": self.downscale_to,
                }
        else:
            payload = {"image_url": self.image_url, "downscale_to": self.downscale_to}
        try:
            response = requests.post(
                f"{self.developers_url}/{self.endpoint}", headers=headers, data=payload
            )
            response_data = response.json()
            return response_data
        except Exception as e:
            print(f"Error: {e}")
            return None

    def remove_background(self, allow_image_id: bool = None):
        headers = {"accept": "application/json", "X-Picsart-API-Key": self.api_key}

        if allow_image_id:
            with open(self.file_image, "rb") as path:
                payload = {
                    "image": path,
                    "image_id": self.image_id,
                    "output_type": self.output_type,
                    "bg_image": self.bg_image,
                    "bg_image_url": self.bg_image_url,
                    "bg_image_id": self.bg_image_id,
                    "bg_color": self.bg_color,
                    "bg_blur": self.bg_blur,
                    "bg_width": self.bg_width,
                    "bg_height": self.bg_height,
                    "scale": self.scale,
                    "format": self.format,
                }
        else:
            payload = {
                "image_url": self.image_url,
                "output_type": self.output_type,
                "scale": self.scale,
                "format": self.format,
            }

        try:
            response = requests.post(
                f"{self.developers_url}/{self.endpoint}", headers=headers, data=payload
            )
            response_data = response.json()
            return response_data
        except Exception as e:
            print(f"Error: {e}")
            return None



from datetime import datetime as dt

import requests

from RyuzakiLib.api.reqs import AsyicXSearcher

owner_base = f"""
Your name is Randy Dev. A kind and friendly AI assistant that answers in
a short and concise answer. Give short step-by-step reasoning if required.

- Powered by @xtdevs on telegram
Today is {dt.now():%A %d %B %Y %H:%M}
"""

class CloudFlare:
    @staticmethod
    async def run(prompt, account_id, api_token):
        payload = {
            "messages": [
                {"role": "system", "content": owner_base},
                {"role": "user", "content": prompt}
            ],
        }
        if account_id and api_token:
            API_BASE_URL = f"https://api.cloudflare.com/client/v4/accounts/{account_id}/ai/run/@cf/meta/llama-3-8b-instruct"
            headers = {"Authorization": f"Bearer {api_token}"}
            response = requests.post(f"{API_BASE_URL}", headers=headers, json=payload)
            if response.status_code != 200:
                return None
            return response.json()["result"].get("response")
        else:
            return "Required account_id and api_token"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

from typing import List, Optional, Union

import requests


class SibylBan:
    @staticmethod
    def _make_request(method: str, url: str, params: dict = None, json_data: dict = None):
        headers = {"accept": "application/json", "api-key": "6398769dabd9fe0e49bedce0354b40a9b1a69d9594dc9d48c1d8a2a071c51e89"}
        try:
            response = requests.request(
                method, url, headers=headers, params=params, json=json_data
            )
            response.raise_for_status()  # Raise an error for bad status codes
            return response.json()
        except requests.RequestException as e:
            print(f"Request failed: {e}")
            return None

    @staticmethod
    def ban(user_id: int = None, reason: str = None) -> str:
        url = "https://akeno.randydev.my.id/sibylban"
        payload = {"user_id": user_id, "reason": reason}
        response = SibylBan._make_request("POST", url, json_data=payload)
        return response.get("randydev", {}).get(
            "message", response.get("message", "Unknown error")
        )

    @staticmethod
    def banlist(user_id: int = None) -> Union[dict, str]:
        url = "https://akeno.randydev.my.id/ryuzaki/sibyl"
        payload = {"user_id": user_id}
        return SibylBan._make_request("GET", url, json_data=payload)

    @staticmethod
    def unban(user_id: int = None) -> Union[dict, str]:
        url = "https://akeno.randydev.my.id/ryuzaki/sibyldel"
        payload = {"user_id": user_id}
        return SibylBan._make_request("DELETE", url, json_data=payload)

    @staticmethod
    def banlist_all() -> Union[dict, str]:
        url = "https://akeno.randydev.my.id/ryuzaki/getbanlist"
        return SibylBan._make_request("GET", url)



import aiohttp


class AsyicXSearcher:
    """
    A class to handle asynchronous HTTP requests using aiohttp.

    This class provides static methods to perform GET, POST, and HEAD requests asynchronously.
    """

    def init(self):
        pass

    @staticmethod
    async def search(
        url: str,
        post: bool = False,
        head: bool = False,
        headers: dict = None,
        evaluate=None,
        object: bool = False,
        re_json: bool = False,
        re_content: bool = False,
        *args,
        **kwargs,
    ):
        """
        Perform an asynchronous HTTP request.

        Args:
            url (str): The URL to send the request to.
            post (bool): If True, perform a POST request. Defaults to False (GET request).
            head (bool): If True, perform a HEAD request. Defaults to False.
            headers (dict): Headers to include in the request. Defaults to None.
            evaluate (callable): A function to evaluate the response. Defaults to None.
            object (bool): If True, return the response object. Defaults to False.
            re_json (bool): If True, return the response in JSON format. Defaults to False.
            re_content (bool): If True, return the raw response content. Defaults to False.
            *args: Additional arguments to pass to the request method.
            **kwargs: Additional keyword arguments to pass to the request method.

        Returns:
            The result of the request, formatted according to the specified flags.

        Raises:
            DependencyMissingError: If aiohttp is not installed.
        """
        if aiohttp:
            async with aiohttp.ClientSession(headers=headers) as session:
                method = (
                    session.head if head else (session.post if post else session.get)
                )
                async with method(url, *args, **kwargs) as response:
                    if evaluate:
                        return await evaluate(response)
                    if re_json:
                        return await response.json()
                    if re_content:
                        return await response.read()
                    if head or object:
                        return response
                    return await response.text()
        else:
            raise DependencyMissingError("Install 'aiohttp' to use this.")

async def async_search(
    url: str,
    post: bool = False,
    head: bool = False,
    headers: dict = None,
    evaluate=None,
    object: bool = False,
    re_json: bool = False,
    re_content: bool = False,
    *args,
    **kwargs,
):

    if aiohttp:
        async with aiohttp.ClientSession(headers=headers) as session:
            method = (
                session.head if head else (session.post if post else session.get)
                )
            async with method(url, *args, **kwargs) as response:
                if evaluate:
                    return await evaluate(response)
                if re_json:
                    return await response.json()
                if re_content:
                    return await response.read()
                if head or object:
                    return response
                return await response.text()
    else:
        raise DependencyMissingError("Install 'aiohttp' to use this.")



from RyuzakiLib.api.reqs import AsyicXSearcher


class SearchAPI:
    def __init__(self, api_key="6398769dabd9fe0e49bedce0354b40a9b1a69d9594dc9d48c1d8a2a071c51e89"):
        self.url = "https://randydev-ryuzaki-api.hf.space/ryuzaki"
        self.api_key = api_key
        self.headers = {"accept": "application/json", "api-key": self.api_key}

    async def custom(self, api_name, post=False, payload=None):
        response = await AsyicXSearcher.search(
            self.url + f"/{api_name}",
            post=post,
            headers=self.headers,
            re_json=True,
            json=payload
        )
        return response



import wget

from RyuzakiLib.api.reqs import async_search


class PornoHub:
    def __init__(self, token=None, base_api_dev: str = "https://akeno.randydev.my.id"):
        self.base_api_dev = base_api_dev
        self.token = token

    async def x_search(self, query=None):
        url = f"{self.base_api_dev}/akeno/xnxxsearch-v2?query={query}&api_key={self.token}"
        res = await async_search(url, re_json=True)
        results = res["randydev"]["results"]
        y = res["randydev"]["results"][0]
        link = y["link"]
        title = y["title"]
        return [link, title, results]

    async def x_download(self, query=None, url=None, is_stream=False):
        if is_stream and url:
            url_ = f"{self.base_api_dev}/akeno/xnxx-dl-v2?link={url}&api_key={self.token}"
            response = await async_search(url_, re_json=True)
            file_path = wget.download(response["randydev"]["results"]["url"])
            thumb = wget.download(response["randydev"]["results"]["thumb"])
            title = response["randydev"]["results"].get("title", "Powered by Randydev")
            return file_path, thumb, title
        else:
            schub = await self.x_search(query=query)
            url_dl = f"{self.base_api_dev}/akeno/xnxx-dl-v2?link={schub[0]}&api_key={self.token}"
            response = await async_search(url_dl, re_json=True)
            file_path = wget.download(response["randydev"]["results"]["url"])
            thumb = wget.download(response["randydev"]["results"]["thumb"])
            title = response["randydev"]["results"].get("title", "Powered by Randydev")
            return file_path, thumb, title



from pyrogram import Client, enums


class PyrogramMod:
    @staticmethod
    async def chat_photos(client: Client, user, set_limit=None, is_limit: bool = False):
        if not isinstance(client, Client):
            raise ValueError("Invalid client instance provided")

        if is_limit and set_limit:
            async for photo in client.get_chat_photos(chat_id=user, limit=set_limit):
                yield photo
        else:
            async for photo in client.get_chat_photos(chat_id=user):
                yield photo

    @staticmethod
    async def chat_members(
        client: Client,
        chat=None,
        query=None,
        filter: enums.ChatMembersFilter = None,
        is_query: bool = False,
        is_filter: bool = False,
    ):
        if not isinstance(client, Client):
            raise ValueError("Invalid client instance provided")

        if is_query and query:
            async for member in client.get_chat_members(chat_id=chat, query=query):
                yield member
        elif is_filter and filter:
            async for member in client.get_chat_members(chat_id=chat, filter=filter):
                yield member
        else:
            async for member in client.get_chat_members(chat_id=chat):
                yield member



import datetime
import os
import signal
import sys
import warnings
from typing import Optional

import openai
import requests
from openai.upload_progress import BufferReader
from openai.validators import (
    apply_necessary_remediation,
    apply_validators,
    get_validators,
    read_any_format,
    write_out_file,
)


class bcolors:
    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


def organization_info(obj):
    organization = getattr(obj, "organization", None)
    if organization is not None:
        return "[organization={}] ".format(organization)
    else:
        return ""


def display(obj):
    sys.stderr.write(organization_info(obj))
    sys.stderr.flush()
    print(obj)


def display_error(e):
    extra = " (HTTP status code: {})".format(e.http_status) if e.http_status is not None else ""
    sys.stderr.write(
        "{}{}Error:{} {}{}\n".format(organization_info(e), bcolors.FAIL, bcolors.ENDC, e, extra)
    )


class Engine:
    @classmethod
    def get(cls, args):
        engine = openai.Engine.retrieve(id=args.id)
        display(engine)

    @classmethod
    def update(cls, args):
        engine = openai.Engine.modify(args.id, replicas=args.replicas)
        display(engine)

    @classmethod
    def generate(cls, args):
        warnings.warn("Engine.generate is deprecated, use Completion.create", DeprecationWarning)
        if args.completions and args.completions > 1 and args.stream:
            raise ValueError("Can't stream multiple completions with openai CLI")

        kwargs = {}
        if args.model is not None:
            kwargs["model"] = args.model
        resp = openai.Engine(id=args.id).generate(
            completions=args.completions,
            context=args.context,
            length=args.length,
            stream=args.stream,
            temperature=args.temperature,
            top_p=args.top_p,
            logprobs=args.logprobs,
            stop=args.stop,
            **kwargs,
        )
        if not args.stream:
            resp = [resp]

        for part in resp:
            completions = len(part["data"])
            for c_idx, c in enumerate(part["data"]):
                if completions > 1:
                    sys.stdout.write("===== Completion {} =====\n".format(c_idx))
                sys.stdout.write("".join(c["text"]))
                if completions > 1:
                    sys.stdout.write("\n")
                sys.stdout.flush()

    @classmethod
    def list(cls, args):
        engines = openai.Engine.list()
        display(engines)


class ChatCompletion:
    @classmethod
    def create(cls, args):
        if args.n is not None and args.n > 1 and args.stream:
            raise ValueError("Can't stream chat completions with n>1 with the current CLI")

        messages = [{"role": role, "content": content} for role, content in args.message]

        resp = openai.ChatCompletion.create(
            # Required
            model=args.model,
            engine=args.engine,
            messages=messages,
            # Optional
            n=args.n,
            max_tokens=args.max_tokens,
            temperature=args.temperature,
            top_p=args.top_p,
            stop=args.stop,
            stream=args.stream,
        )
        if not args.stream:
            resp = [resp]

        for part in resp:
            choices = part["choices"]
            for c_idx, c in enumerate(sorted(choices, key=lambda s: s["index"])):
                if len(choices) > 1:
                    sys.stdout.write("===== Chat Completion {} =====\n".format(c_idx))
                if args.stream:
                    delta = c["delta"]
                    if "content" in delta:
                        sys.stdout.write(delta["content"])
                else:
                    sys.stdout.write(c["message"]["content"])
                    if len(choices) > 1:  # not in streams
                        sys.stdout.write("\n")
                sys.stdout.flush()


class Completion:
    @classmethod
    def create(cls, args):
        if args.n is not None and args.n > 1 and args.stream:
            raise ValueError("Can't stream completions with n>1 with the current CLI")

        if args.engine and args.model:
            warnings.warn("In most cases, you should not be specifying both engine and model.")

        resp = openai.Completion.create(
            engine=args.engine,
            model=args.model,
            n=args.n,
            max_tokens=args.max_tokens,
            logprobs=args.logprobs,
            prompt=args.prompt,
            stream=args.stream,
            temperature=args.temperature,
            top_p=args.top_p,
            stop=args.stop,
            echo=True,
        )
        if not args.stream:
            resp = [resp]

        for part in resp:
            choices = part["choices"]
            for c_idx, c in enumerate(sorted(choices, key=lambda s: s["index"])):
                if len(choices) > 1:
                    sys.stdout.write("===== Completion {} =====\n".format(c_idx))
                sys.stdout.write(c["text"])
                if len(choices) > 1:
                    sys.stdout.write("\n")
                sys.stdout.flush()


class Deployment:
    @classmethod
    def get(cls, args):
        resp = openai.Deployment.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        model = openai.Deployment.delete(args.id)
        print(model)

    @classmethod
    def list(cls, args):
        models = openai.Deployment.list()
        print(models)

    @classmethod
    def create(cls, args):
        models = openai.Deployment.create(
            model=args.model, scale_settings={"scale_type": args.scale_type}
        )
        print(models)


class Model:
    @classmethod
    def get(cls, args):
        resp = openai.Model.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        model = openai.Model.delete(args.id)
        print(model)

    @classmethod
    def list(cls, args):
        models = openai.Model.list()
        print(models)


class File:
    @classmethod
    def create(cls, args):
        with open(args.file, "rb") as file_reader:
            buffer_reader = BufferReader(file_reader.read(), desc="Upload progress")
        resp = openai.File.create(
            file=buffer_reader,
            purpose=args.purpose,
            user_provided_filename=args.file,
        )
        print(resp)

    @classmethod
    def get(cls, args):
        resp = openai.File.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        file = openai.File.delete(args.id)
        print(file)

    @classmethod
    def list(cls, args):
        file = openai.File.list()
        print(file)


class Image:
    @classmethod
    def create(cls, args):
        resp = openai.Image.create(
            prompt=args.prompt,
            size=args.size,
            n=args.num_images,
            response_format=args.response_format,
        )
        print(resp)

    @classmethod
    def create_variation(cls, args):
        with open(args.image, "rb") as file_reader:
            buffer_reader = BufferReader(file_reader.read(), desc="Upload progress")
        resp = openai.Image.create_variation(
            image=buffer_reader,
            size=args.size,
            n=args.num_images,
            response_format=args.response_format,
        )
        print(resp)

    @classmethod
    def create_edit(cls, args):
        with open(args.image, "rb") as file_reader:
            image_reader = BufferReader(file_reader.read(), desc="Upload progress")
        mask_reader = None
        if args.mask is not None:
            with open(args.mask, "rb") as file_reader:
                mask_reader = BufferReader(file_reader.read(), desc="Upload progress")
        resp = openai.Image.create_edit(
            image=image_reader,
            mask=mask_reader,
            prompt=args.prompt,
            size=args.size,
            n=args.num_images,
            response_format=args.response_format,
        )
        print(resp)


class Audio:
    @classmethod
    def transcribe(cls, args):
        with open(args.file, "rb") as r:
            file_reader = BufferReader(r.read(), desc="Upload progress")

        resp = openai.Audio.transcribe_raw(
            # Required
            model=args.model,
            file=file_reader,
            filename=args.file,
            # Optional
            response_format=args.response_format,
            language=args.language,
            temperature=args.temperature,
            prompt=args.prompt,
        )
        print(resp)

    @classmethod
    def translate(cls, args):
        with open(args.file, "rb") as r:
            file_reader = BufferReader(r.read(), desc="Upload progress")
        resp = openai.Audio.translate_raw(
            # Required
            model=args.model,
            file=file_reader,
            filename=args.file,
            # Optional
            response_format=args.response_format,
            language=args.language,
            temperature=args.temperature,
            prompt=args.prompt,
        )
        print(resp)


class FineTune:
    @classmethod
    def list(cls, args):
        resp = openai.FineTune.list()
        print(resp)

    @classmethod
    def _is_url(cls, file: str):
        return file.lower().startswith("http")

    @classmethod
    def _download_file_from_public_url(cls, url: str) -> Optional[bytes]:
        resp = requests.get(url)
        if resp.status_code == 200:
            return resp.content
        else:
            return None

    @classmethod
    def _maybe_upload_file(
        cls,
        file: Optional[str] = None,
        content: Optional[bytes] = None,
        user_provided_file: Optional[str] = None,
        check_if_file_exists: bool = True,
    ):
        # Exactly one of `file` or `content` must be provided
        if (file is None) == (content is None):
            raise ValueError("Exactly one of `file` or `content` must be provided")

        if content is None:
            assert file is not None
            with open(file, "rb") as f:
                content = f.read()

        if check_if_file_exists:
            bytes = len(content)
            matching_files = openai.File.find_matching_files(
                name=user_provided_file or f.name, bytes=bytes, purpose="fine-tune"
            )
            if len(matching_files) > 0:
                file_ids = [f["id"] for f in matching_files]
                sys.stdout.write(
                    "Found potentially duplicated files with name '{name}', purpose 'fine-tune' and size {size} bytes\n".format(
                        name=os.path.basename(matching_files[0]["filename"]),
                        size=matching_files[0]["bytes"]
                        if "bytes" in matching_files[0]
                        else matching_files[0]["size"],
                    )
                )
                sys.stdout.write("\n".join(file_ids))
                while True:
                    sys.stdout.write(
                        "\nEnter file ID to reuse an already uploaded file, or an empty string to upload this file anyway: "
                    )
                    inp = sys.stdin.readline().strip()
                    if inp in file_ids:
                        sys.stdout.write("Reusing already uploaded file: {id}\n".format(id=inp))
                        return inp
                    elif inp == "":
                        break
                    else:
                        sys.stdout.write(
                            "File id '{id}' is not among the IDs of the potentially duplicated files\n".format(
                                id=inp
                            )
                        )

        buffer_reader = BufferReader(content, desc="Upload progress")
        resp = openai.File.create(
            file=buffer_reader,
            purpose="fine-tune",
            user_provided_filename=user_provided_file or file,
        )
        sys.stdout.write(
            "Uploaded file from {file}: {id}\n".format(
                file=user_provided_file or file, id=resp["id"]
            )
        )
        return resp["id"]

    @classmethod
    def _get_or_upload(cls, file, check_if_file_exists=True):
        try:
            # 1. If it's a valid file, use it
            openai.File.retrieve(file)
            return file
        except openai.error.InvalidRequestError:
            pass
        if os.path.isfile(file):
            # 2. If it's a file on the filesystem, upload it
            return cls._maybe_upload_file(file=file, check_if_file_exists=check_if_file_exists)
        if cls._is_url(file):
            # 3. If it's a URL, download it temporarily
            content = cls._download_file_from_public_url(file)
            if content is not None:
                return cls._maybe_upload_file(
                    content=content,
                    check_if_file_exists=check_if_file_exists,
                    user_provided_file=file,
                )
        return file

    @classmethod
    def create(cls, args):
        create_args = {
            "training_file": cls._get_or_upload(args.training_file, args.check_if_files_exist),
        }
        if args.validation_file:
            create_args["validation_file"] = cls._get_or_upload(
                args.validation_file, args.check_if_files_exist
            )

        for hparam in (
            "model",
            "suffix",
            "n_epochs",
            "batch_size",
            "learning_rate_multiplier",
            "prompt_loss_weight",
            "compute_classification_metrics",
            "classification_n_classes",
            "classification_positive_class",
            "classification_betas",
        ):
            attr = getattr(args, hparam)
            if attr is not None:
                create_args[hparam] = attr

        resp = openai.FineTune.create(**create_args)

        if args.no_follow:
            print(resp)
            return

        sys.stdout.write(
            "Created fine-tune: {job_id}\n"
            "Streaming events until fine-tuning is complete...\n\n"
            "(Ctrl-C will interrupt the stream, but not cancel the fine-tune)\n".format(
                job_id=resp["id"]
            )
        )
        cls._stream_events(resp["id"])

    @classmethod
    def get(cls, args):
        resp = openai.FineTune.retrieve(id=args.id)
        print(resp)

    @classmethod
    def results(cls, args):
        fine_tune = openai.FineTune.retrieve(id=args.id)
        if "result_files" not in fine_tune or len(fine_tune["result_files"]) == 0:
            raise openai.error.InvalidRequestError(
                f"No results file available for fine-tune {args.id}", "id"
            )
        result_file = openai.FineTune.retrieve(id=args.id)["result_files"][0]
        resp = openai.File.download(id=result_file["id"])
        print(resp.decode("utf-8"))

    @classmethod
    def events(cls, args):
        if args.stream:
            raise openai.error.OpenAIError(
                message=(
                    "The --stream parameter is deprecated, use fine_tunes.follow "
                    "instead:\n\n"
                    "  openai api fine_tunes.follow -i {id}\n".format(id=args.id)
                ),
            )

        resp = openai.FineTune.list_events(id=args.id)  # type: ignore
        print(resp)

    @classmethod
    def follow(cls, args):
        cls._stream_events(args.id)

    @classmethod
    def _stream_events(cls, job_id):
        def signal_handler(sig, frame):
            status = openai.FineTune.retrieve(job_id).status
            sys.stdout.write(
                "\nStream interrupted. Job is still {status}.\n"
                "To resume the stream, run:\n\n"
                "  openai api fine_tunes.follow -i {job_id}\n\n"
                "To cancel your job, run:\n\n"
                "  openai api fine_tunes.cancel -i {job_id}\n\n".format(
                    status=status, job_id=job_id
                )
            )
            sys.exit(0)

        signal.signal(signal.SIGINT, signal_handler)

        events = openai.FineTune.stream_events(job_id)
        # TODO(rachel): Add a nifty spinner here.
        try:
            for event in events:
                sys.stdout.write(
                    "[%s] %s"
                    % (
                        datetime.datetime.fromtimestamp(event["created_at"]),
                        event["message"],
                    )
                )
                sys.stdout.write("\n")
                sys.stdout.flush()
        except Exception:
            sys.stdout.write(
                "\nStream interrupted (client disconnected).\n"
                "To resume the stream, run:\n\n"
                "  openai api fine_tunes.follow -i {job_id}\n\n".format(job_id=job_id)
            )
            return

        resp = openai.FineTune.retrieve(id=job_id)
        status = resp["status"]
        if status == "succeeded":
            sys.stdout.write("\nJob complete! Status: succeeded 🎉")
            sys.stdout.write(
                "\nTry out your fine-tuned model:\n\n"
                "openai api completions.create -m {model} -p <YOUR_PROMPT>".format(
                    model=resp["fine_tuned_model"]
                )
            )
        elif status == "failed":
            sys.stdout.write(
                "\nJob failed. Please contact us through our help center at help.openai.com if you need assistance."
            )
        sys.stdout.write("\n")

    @classmethod
    def cancel(cls, args):
        resp = openai.FineTune.cancel(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        resp = openai.FineTune.delete(sid=args.id)
        print(resp)

    @classmethod
    def prepare_data(cls, args):
        sys.stdout.write("Analyzing...\n")
        fname = args.file
        auto_accept = args.quiet
        df, remediation = read_any_format(fname)
        apply_necessary_remediation(None, remediation)

        validators = get_validators()

        apply_validators(
            df,
            fname,
            remediation,
            validators,
            auto_accept,
            write_out_file_func=write_out_file,
        )


class FineTuningJob:
    @classmethod
    def list(cls, args):
        has_ft_jobs = False
        for fine_tune_job in openai.FineTuningJob.auto_paging_iter():
            has_ft_jobs = True
            print(fine_tune_job)
        if not has_ft_jobs:
            print("No fine-tuning jobs found.")

    @classmethod
    def _is_url(cls, file: str):
        return file.lower().startswith("http")

    @classmethod
    def _download_file_from_public_url(cls, url: str) -> Optional[bytes]:
        resp = requests.get(url)
        if resp.status_code == 200:
            return resp.content
        else:
            return None

    @classmethod
    def _maybe_upload_file(
        cls,
        file: Optional[str] = None,
        content: Optional[bytes] = None,
        user_provided_file: Optional[str] = None,
        check_if_file_exists: bool = True,
    ):
        # Exactly one of `file` or `content` must be provided
        if (file is None) == (content is None):
            raise ValueError("Exactly one of `file` or `content` must be provided")

        if content is None:
            assert file is not None
            with open(file, "rb") as f:
                content = f.read()

        if check_if_file_exists:
            bytes = len(content)
            matching_files = openai.File.find_matching_files(
                name=user_provided_file or f.name,
                bytes=bytes,
                purpose="fine-tune",
            )
            if len(matching_files) > 0:
                file_ids = [f["id"] for f in matching_files]
                sys.stdout.write(
                    "Found potentially duplicated files with name '{name}', purpose 'fine-tune', and size {size} bytes\n".format(
                        name=os.path.basename(matching_files[0]["filename"]),
                        size=matching_files[0]["bytes"]
                        if "bytes" in matching_files[0]
                        else matching_files[0]["size"],
                    )
                )
                sys.stdout.write("\n".join(file_ids))
                while True:
                    sys.stdout.write(
                        "\nEnter file ID to reuse an already uploaded file, or an empty string to upload this file anyway: "
                    )
                    inp = sys.stdin.readline().strip()
                    if inp in file_ids:
                        sys.stdout.write("Reusing already uploaded file: {id}\n".format(id=inp))
                        return inp
                    elif inp == "":
                        break
                    else:
                        sys.stdout.write(
                            "File id '{id}' is not among the IDs of the potentially duplicated files\n".format(
                                id=inp
                            )
                        )

        buffer_reader = BufferReader(content, desc="Upload progress")
        resp = openai.File.create(
            file=buffer_reader,
            purpose="fine-tune",
            user_provided_filename=user_provided_file or file,
        )
        sys.stdout.write(
            "Uploaded file from {file}: {id}\n".format(
                file=user_provided_file or file, id=resp["id"]
            )
        )
        sys.stdout.write("Waiting for file to finish processing before proceeding..\n")
        sys.stdout.flush()
        status = openai.File.wait_for_processing(resp["id"])
        if status != "processed":
            raise openai.error.OpenAIError(
                "File {id} failed to process, status={status}.".format(
                    id=resp["id"], status=status
                )
            )

        sys.stdout.write(
            "File {id} finished processing and is ready for use in fine-tuning".format(
                id=resp["id"]
            )
        )
        sys.stdout.flush()
        return resp["id"]

    @classmethod
    def _get_or_upload(cls, file, check_if_file_exists=True):
        try:
            # 1. If it's a valid file, use it
            openai.File.retrieve(file)
            return file
        except openai.error.InvalidRequestError:
            pass
        if os.path.isfile(file):
            # 2. If it's a file on the filesystem, upload it
            return cls._maybe_upload_file(file=file, check_if_file_exists=check_if_file_exists)
        if cls._is_url(file):
            # 3. If it's a URL, download it temporarily
            content = cls._download_file_from_public_url(file)
            if content is not None:
                return cls._maybe_upload_file(
                    content=content,
                    check_if_file_exists=check_if_file_exists,
                    user_provided_file=file,
                )
        return file

    @classmethod
    def create(cls, args):
        create_args = {
            "training_file": cls._get_or_upload(args.training_file, args.check_if_files_exist),
        }
        if args.validation_file:
            create_args["validation_file"] = cls._get_or_upload(
                args.validation_file, args.check_if_files_exist
            )

        for param in ("model", "suffix"):
            attr = getattr(args, param)
            if attr is not None:
                create_args[param] = attr

        if getattr(args, "n_epochs"):
            create_args["hyperparameters"] = {
                "n_epochs": args.n_epochs,
            }

        resp = openai.FineTuningJob.create(**create_args)
        print(resp)
        return

    @classmethod
    def get(cls, args):
        resp = openai.FineTuningJob.retrieve(id=args.id)
        print(resp)

    @classmethod
    def results(cls, args):
        fine_tune = openai.FineTuningJob.retrieve(id=args.id)
        if "result_files" not in fine_tune or len(fine_tune["result_files"]) == 0:
            raise openai.error.InvalidRequestError(
                f"No results file available for fine-tune {args.id}", "id"
            )
        result_file = openai.FineTuningJob.retrieve(id=args.id)["result_files"][0]
        resp = openai.File.download(id=result_file)
        print(resp.decode("utf-8"))

    @classmethod
    def events(cls, args):
        seen, has_more, after = 0, True, None
        while has_more:
            resp = openai.FineTuningJob.list_events(id=args.id, after=after)  # type: ignore
            for event in resp["data"]:
                print(event)
                seen += 1
                if args.limit is not None and seen >= args.limit:
                    return
            has_more = resp.get("has_more", False)
            if resp["data"]:
                after = resp["data"][-1]["id"]

    @classmethod
    def follow(cls, args):
        raise openai.error.OpenAIError(
            message="Event streaming is not yet supported for `fine_tuning.job` events"
        )

    @classmethod
    def cancel(cls, args):
        resp = openai.FineTuningJob.cancel(id=args.id)
        print(resp)


class WandbLogger:
    @classmethod
    def sync(cls, args):
        import openai.wandb_logger

        resp = openai.wandb_logger.WandbLogger.sync(
            id=args.id,
            n_fine_tunes=args.n_fine_tunes,
            project=args.project,
            entity=args.entity,
            force=args.force,
        )
        print(resp)


def tools_register(parser):
    subparsers = parser.add_subparsers(title="Tools", help="Convenience client side tools")

    def help(args):
        parser.print_help()

    parser.set_defaults(func=help)

    sub = subparsers.add_parser("fine_tunes.prepare_data")
    sub.add_argument(
        "-f",
        "--file",
        required=True,
        help="JSONL, JSON, CSV, TSV, TXT or XLSX file containing prompt-completion examples to be analyzed."
        "This should be the local file path.",
    )
    sub.add_argument(
        "-q",
        "--quiet",
        required=False,
        action="store_true",
        help="Auto accepts all suggestions, without asking for user input. To be used within scripts.",
    )
    sub.set_defaults(func=FineTune.prepare_data)


def api_register(parser):
    # Engine management
    subparsers = parser.add_subparsers(help="All API subcommands")

    def help(args):
        parser.print_help()

    parser.set_defaults(func=help)

    sub = subparsers.add_parser("engines.list")
    sub.set_defaults(func=Engine.list)

    sub = subparsers.add_parser("engines.get")
    sub.add_argument("-i", "--id", required=True)
    sub.set_defaults(func=Engine.get)

    sub = subparsers.add_parser("engines.update")
    sub.add_argument("-i", "--id", required=True)
    sub.add_argument("-r", "--replicas", type=int)
    sub.set_defaults(func=Engine.update)

    sub = subparsers.add_parser("engines.generate")
    sub.add_argument("-i", "--id", required=True)
    sub.add_argument("--stream", help="Stream tokens as they're ready.", action="store_true")
    sub.add_argument("-c", "--context", help="An optional context to generate from")
    sub.add_argument("-l", "--length", help="How many tokens to generate", type=int)
    sub.add_argument(
        "-t",
        "--temperature",
        help="""What sampling temperature to use. Higher values means the model will take more risks. Try 0.9 for more creative applications, and 0 (argmax sampling) for ones with a well-defined answer.

Mutually exclusive with `top_p`.""",
        type=float,
    )
    sub.add_argument(
        "-p",
        "--top_p",
        help="""An alternative to sampling with temperature, called nucleus sampling, where the considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10%% probability mass are considered.

            Mutually exclusive with `temperature`.""",
        type=float,
    )
    sub.add_argument(
        "-n",
        "--completions",
        help="How many parallel completions to run on this context",
        type=int,
    )
    sub.add_argument(
        "--logprobs",
        help="Include the log probabilites on the `logprobs` most likely tokens. So for example, if `logprobs` is 10, the API will return a list of the 10 most likely tokens. If `logprobs` is supplied, the API will always return the logprob of the generated token, so there may be up to `logprobs+1` elements in the response.",
        type=int,
    )
    sub.add_argument("--stop", help="A stop sequence at which to stop generating tokens.")
    sub.add_argument(
        "-m",
        "--model",
        required=False,
        help="A model (most commonly a model ID) to generate from. Defaults to the engine's default model.",
    )
    sub.set_defaults(func=Engine.generate)

    # Chat Completions
    sub = subparsers.add_parser("chat_completions.create")

    sub._action_groups.pop()
    req = sub.add_argument_group("required arguments")
    opt = sub.add_argument_group("optional arguments")

    req.add_argument(
        "-g",
        "--message",
        action="append",
        nargs=2,
        metavar=("ROLE", "CONTENT"),
        help="A message in `{role} {content}` format. Use this argument multiple times to add multiple messages.",
        required=True,
    )

    group = opt.add_mutually_exclusive_group()
    group.add_argument(
        "-e",
        "--engine",
        help="The engine to use. See https://learn.microsoft.com/en-us/azure/cognitive-services/openai/chatgpt-quickstart?pivots=programming-language-python for more about what engines are available.",
    )
    group.add_argument(
        "-m",
        "--model",
        help="The model to use.",
    )

    opt.add_argument(
        "-n",
        "--n",
        help="How many completions to generate for the conversation.",
        type=int,
    )
    opt.add_argument(
        "-M", "--max-tokens", help="The maximum number of tokens to generate.", type=int
    )
    opt.add_argument(
        "-t",
        "--temperature",
        help="""What sampling temperature to use. Higher values means the model will take more risks. Try 0.9 for more creative applications, and 0 (argmax sampling) for ones with a well-defined answer.

Mutually exclusive with `top_p`.""",
        type=float,
    )
    opt.add_argument(
        "-P",
        "--top_p",
        help="""An alternative to sampling with temperature, called nucleus sampling, where the considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10%% probability mass are considered.

            Mutually exclusive with `temperature`.""",
        type=float,
    )
    opt.add_argument(
        "--stop",
        help="A stop sequence at which to stop generating tokens for the message.",
    )
    opt.add_argument("--stream", help="Stream messages as they're ready.", action="store_true")
    sub.set_defaults(func=ChatCompletion.create)

    # Completions
    sub = subparsers.add_parser("completions.create")
    sub.add_argument(
        "-e",
        "--engine",
        help="The engine to use. See https://platform.openai.com/docs/engines for more about what engines are available.",
    )
    sub.add_argument(
        "-m",
        "--model",
        help="The model to use. At most one of `engine` or `model` should be specified.",
    )
    sub.add_argument("--stream", help="Stream tokens as they're ready.", action="store_true")
    sub.add_argument("-p", "--prompt", help="An optional prompt to complete from")
    sub.add_argument(
        "-M", "--max-tokens", help="The maximum number of tokens to generate", type=int
    )
    sub.add_argument(
        "-t",
        "--temperature",
        help="""What sampling temperature to use. Higher values means the model will take more risks. Try 0.9 for more creative applications, and 0 (argmax sampling) for ones with a well-defined answer.

Mutually exclusive with `top_p`.""",
        type=float,
    )
    sub.add_argument(
        "-P",
        "--top_p",
        help="""An alternative to sampling with temperature, called nucleus sampling, where the considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10%% probability mass are considered.

            Mutually exclusive with `temperature`.""",
        type=float,
    )
    sub.add_argument(
        "-n",
        "--n",
        help="How many sub-completions to generate for each prompt.",
        type=int,
    )
    sub.add_argument(
        "--logprobs",
        help="Include the log probabilites on the `logprobs` most likely tokens, as well the chosen tokens. So for example, if `logprobs` is 10, the API will return a list of the 10 most likely tokens. If `logprobs` is 0, only the chosen tokens will have logprobs returned.",
        type=int,
    )
    sub.add_argument("--stop", help="A stop sequence at which to stop generating tokens.")
    sub.set_defaults(func=Completion.create)

    # Deployments
    sub = subparsers.add_parser("deployments.list")
    sub.set_defaults(func=Deployment.list)

    sub = subparsers.add_parser("deployments.get")
    sub.add_argument("-i", "--id", required=True, help="The deployment ID")
    sub.set_defaults(func=Deployment.get)

    sub = subparsers.add_parser("deployments.delete")
    sub.add_argument("-i", "--id", required=True, help="The deployment ID")
    sub.set_defaults(func=Deployment.delete)

    sub = subparsers.add_parser("deployments.create")
    sub.add_argument("-m", "--model", required=True, help="The model ID")
    sub.add_argument(
        "-s",
        "--scale_type",
        required=True,
        help="The scale type. Either 'manual' or 'standard'",
    )
    sub.set_defaults(func=Deployment.create)

    # Models
    sub = subparsers.add_parser("models.list")
    sub.set_defaults(func=Model.list)

    sub = subparsers.add_parser("models.get")
    sub.add_argument("-i", "--id", required=True, help="The model ID")
    sub.set_defaults(func=Model.get)

    sub = subparsers.add_parser("models.delete")
    sub.add_argument("-i", "--id", required=True, help="The model ID")
    sub.set_defaults(func=Model.delete)

    # Files
    sub = subparsers.add_parser("files.create")

    sub.add_argument(
        "-f",
        "--file",
        required=True,
        help="File to upload",
    )
    sub.add_argument(
        "-p",
        "--purpose",
        help="Why are you uploading this file? (see https://platform.openai.com/docs/api-reference/ for purposes)",
        required=True,
    )
    sub.set_defaults(func=File.create)

    sub = subparsers.add_parser("files.get")
    sub.add_argument("-i", "--id", required=True, help="The files ID")
    sub.set_defaults(func=File.get)

    sub = subparsers.add_parser("files.delete")
    sub.add_argument("-i", "--id", required=True, help="The files ID")
    sub.set_defaults(func=File.delete)

    sub = subparsers.add_parser("files.list")
    sub.set_defaults(func=File.list)

    # Finetune
    sub = subparsers.add_parser("fine_tunes.list")
    sub.set_defaults(func=FineTune.list)

    sub = subparsers.add_parser("fine_tunes.create")
    sub.add_argument(
        "-t",
        "--training_file",
        required=True,
        help="JSONL file containing prompt-completion examples for training. This can "
        "be the ID of a file uploaded through the OpenAI API (e.g. file-abcde12345), "
        'a local file path, or a URL that starts with "http".',
    )
    sub.add_argument(
        "-v",
        "--validation_file",
        help="JSONL file containing prompt-completion examples for validation. This can "
        "be the ID of a file uploaded through the OpenAI API (e.g. file-abcde12345), "
        'a local file path, or a URL that starts with "http".',
    )
    sub.add_argument(
        "--no_check_if_files_exist",
        dest="check_if_files_exist",
        action="store_false",
        help="If this argument is set and training_file or validation_file are file paths, immediately upload them. If this argument is not set, check if they may be duplicates of already uploaded files before uploading, based on file name and file size.",
    )
    sub.add_argument(
        "-m",
        "--model",
        help="The model to start fine-tuning from",
    )
    sub.add_argument(
        "--suffix",
        help="If set, this argument can be used to customize the generated fine-tuned model name."
        "All punctuation and whitespace in `suffix` will be replaced with a "
        "single dash, and the string will be lower cased. The max "
        "length of `suffix` is 40 chars. "
        "The generated name will match the form `{base_model}:ft-{org-title}:{suffix}-{timestamp}`. "
        'For example, `openai api fine_tunes.create -t test.jsonl -m ada --suffix "custom model name" '
        "could generate a model with the name "
        "ada:ft-your-org:custom-model-name-2022-02-15-04-21-04",
    )
    sub.add_argument(
        "--no_follow",
        action="store_true",
        help="If set, returns immediately after creating the job. Otherwise, streams events and waits for the job to complete.",
    )
    sub.add_argument(
        "--n_epochs",
        type=int,
        help="The number of epochs to train the model for. An epoch refers to one "
        "full cycle through the training dataset.",
    )
    sub.add_argument(
        "--batch_size",
        type=int,
        help="The batch size to use for training. The batch size is the number of "
        "training examples used to train a single forward and backward pass.",
    )
    sub.add_argument(
        "--learning_rate_multiplier",
        type=float,
        help="The learning rate multiplier to use for training. The fine-tuning "
        "learning rate is determined by the original learning rate used for "
        "pretraining multiplied by this value.",
    )
    sub.add_argument(
        "--prompt_loss_weight",
        type=float,
        help="The weight to use for the prompt loss. The optimum value here depends "
        "depends on your use case. This determines how much the model prioritizes "
        "learning from prompt tokens vs learning from completion tokens.",
    )
    sub.add_argument(
        "--compute_classification_metrics",
        action="store_true",
        help="If set, we calculate classification-specific metrics such as accuracy "
        "and F-1 score using the validation set at the end of every epoch.",
    )
    sub.set_defaults(compute_classification_metrics=None)
    sub.add_argument(
        "--classification_n_classes",
        type=int,
        help="The number of classes in a classification task. This parameter is "
        "required for multiclass classification.",
    )
    sub.add_argument(
        "--classification_positive_class",
        help="The positive class in binary classification. This parameter is needed "
        "to generate precision, recall and F-1 metrics when doing binary "
        "classification.",
    )
    sub.add_argument(
        "--classification_betas",
        type=float,
        nargs="+",
        help="If this is provided, we calculate F-beta scores at the specified beta "
        "values. The F-beta score is a generalization of F-1 score. This is only "
        "used for binary classification.",
    )
    sub.set_defaults(func=FineTune.create)

    sub = subparsers.add_parser("fine_tunes.get")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTune.get)

    sub = subparsers.add_parser("fine_tunes.results")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTune.results)

    sub = subparsers.add_parser("fine_tunes.events")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")

    # TODO(rachel): Remove this in 1.0
    sub.add_argument(
        "-s",
        "--stream",
        action="store_true",
        help="[DEPRECATED] If set, events will be streamed until the job is done. Otherwise, "
        "displays the event history to date.",
    )
    sub.set_defaults(func=FineTune.events)

    sub = subparsers.add_parser("fine_tunes.follow")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTune.follow)

    sub = subparsers.add_parser("fine_tunes.cancel")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTune.cancel)

    sub = subparsers.add_parser("fine_tunes.delete")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTune.delete)

    # Image
    sub = subparsers.add_parser("image.create")
    sub.add_argument("-p", "--prompt", type=str, required=True)
    sub.add_argument("-n", "--num-images", type=int, default=1)
    sub.add_argument(
        "-s", "--size", type=str, default="1024x1024", help="Size of the output image"
    )
    sub.add_argument("--response-format", type=str, default="url")
    sub.set_defaults(func=Image.create)

    sub = subparsers.add_parser("image.create_edit")
    sub.add_argument("-p", "--prompt", type=str, required=True)
    sub.add_argument("-n", "--num-images", type=int, default=1)
    sub.add_argument(
        "-I",
        "--image",
        type=str,
        required=True,
        help="Image to modify. Should be a local path and a PNG encoded image.",
    )
    sub.add_argument(
        "-s", "--size", type=str, default="1024x1024", help="Size of the output image"
    )
    sub.add_argument("--response-format", type=str, default="url")
    sub.add_argument(
        "-M",
        "--mask",
        type=str,
        required=False,
        help="Path to a mask image. It should be the same size as the image you're editing and a RGBA PNG image. The Alpha channel acts as the mask.",
    )
    sub.set_defaults(func=Image.create_edit)

    sub = subparsers.add_parser("image.create_variation")
    sub.add_argument("-n", "--num-images", type=int, default=1)
    sub.add_argument(
        "-I",
        "--image",
        type=str,
        required=True,
        help="Image to modify. Should be a local path and a PNG encoded image.",
    )
    sub.add_argument(
        "-s", "--size", type=str, default="1024x1024", help="Size of the output image"
    )
    sub.add_argument("--response-format", type=str, default="url")
    sub.set_defaults(func=Image.create_variation)

    # Audio
    # transcriptions
    sub = subparsers.add_parser("audio.transcribe")
    # Required
    sub.add_argument("-m", "--model", type=str, default="whisper-1")
    sub.add_argument("-f", "--file", type=str, required=True)
    # Optional
    sub.add_argument("--response-format", type=str)
    sub.add_argument("--language", type=str)
    sub.add_argument("-t", "--temperature", type=float)
    sub.add_argument("--prompt", type=str)
    sub.set_defaults(func=Audio.transcribe)
    # translations
    sub = subparsers.add_parser("audio.translate")
    # Required
    sub.add_argument("-m", "--model", type=str, default="whisper-1")
    sub.add_argument("-f", "--file", type=str, required=True)
    # Optional
    sub.add_argument("--response-format", type=str)
    sub.add_argument("--language", type=str)
    sub.add_argument("-t", "--temperature", type=float)
    sub.add_argument("--prompt", type=str)
    sub.set_defaults(func=Audio.translate)

    # FineTuning Jobs
    sub = subparsers.add_parser("fine_tuning.job.list")
    sub.set_defaults(func=FineTuningJob.list)

    sub = subparsers.add_parser("fine_tuning.job.create")
    sub.add_argument(
        "-t",
        "--training_file",
        required=True,
        help="JSONL file containing either chat-completion or prompt-completion examples for training. "
        "This can be the ID of a file uploaded through the OpenAI API (e.g. file-abcde12345), "
        'a local file path, or a URL that starts with "http".',
    )
    sub.add_argument(
        "-v",
        "--validation_file",
        help="JSONL file containing either chat-completion or prompt-completion examples for validation. "
        "This can be the ID of a file uploaded through the OpenAI API (e.g. file-abcde12345), "
        'a local file path, or a URL that starts with "http".',
    )
    sub.add_argument(
        "--no_check_if_files_exist",
        dest="check_if_files_exist",
        action="store_false",
        help="If this argument is set and training_file or validation_file are file paths, immediately upload them. If this argument is not set, check if they may be duplicates of already uploaded files before uploading, based on file name and file size.",
    )
    sub.add_argument(
        "-m",
        "--model",
        help="The model to start fine-tuning from",
    )
    sub.add_argument(
        "--suffix",
        help="If set, this argument can be used to customize the generated fine-tuned model name."
        "All punctuation and whitespace in `suffix` will be replaced with a "
        "single dash, and the string will be lower cased. The max "
        "length of `suffix` is 18 chars. "
        "The generated name will match the form `ft:{base_model}:{org-title}:{suffix}:{rstring}` where `rstring` "
        "is a random string sortable as a timestamp. "
        'For example, `openai api fine_tuning.job.create -t test.jsonl -m gpt-3.5-turbo-0613 --suffix "first finetune!" '
        "could generate a model with the name "
        "ft:gpt-3.5-turbo-0613:your-org:first-finetune:7p4PqAoY",
    )
    sub.add_argument(
        "--n_epochs",
        type=int,
        help="The number of epochs to train the model for. An epoch refers to one "
        "full cycle through the training dataset.",
    )
    sub.set_defaults(func=FineTuningJob.create)

    sub = subparsers.add_parser("fine_tuning.job.get")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTuningJob.get)

    sub = subparsers.add_parser("fine_tuning.job.results")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTuningJob.results)

    sub = subparsers.add_parser("fine_tuning.job.events")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.add_argument(
        "--limit",
        type=int,
        required=False,
        help="The number of events to return, starting from most recent. If not specified, all events will be returned.",
    )
    sub.set_defaults(func=FineTuningJob.events)

    sub = subparsers.add_parser("fine_tuning.job.follow")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTuningJob.follow)

    sub = subparsers.add_parser("fine_tuning.job.cancel")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
    sub.set_defaults(func=FineTuningJob.cancel)


def wandb_register(parser):
    subparsers = parser.add_subparsers(title="wandb", help="Logging with Weights & Biases")

    def help(args):
        parser.print_help()

    parser.set_defaults(func=help)

    sub = subparsers.add_parser("sync")
    sub.add_argument("-i", "--id", help="The id of the fine-tune job (optional)")
    sub.add_argument(
        "-n",
        "--n_fine_tunes",
        type=int,
        default=None,
        help="Number of most recent fine-tunes to log when an id is not provided. By default, every fine-tune is synced.",
    )
    sub.add_argument(
        "--project",
        default="GPT-3",
        help="""Name of the project where you're sending runs. By default, it is "GPT-3".""",
    )
    sub.add_argument(
        "--entity",
        help="Username or team name where you're sending runs. By default, your default entity is used, which is usually your username.",
    )
    sub.add_argument(
        "--force",
        action="store_true",
        help="Forces logging and overwrite existing wandb run of the same fine-tune.",
    )
    sub.set_defaults(force=False)
    sub.set_defaults(func=WandbLogger.sync)



from pymongo import MongoClient


class MongoConnect:
    def __init__(self, mongo_url=None, client=None, database=None):
        self.mongo_url = mongo_url
        self.client = client
        self.database = database

    def get_collection(self, mongodb_dev: bool = None):
        if mongodb_dev:
            client_mongo = MongoClient(self.mongo_url)
            db = client_mongo[self.client][self.database]
            return db[self.collection_name]
        else:
            client_mongo = MongoClient(self.mongo_url)
            db = client_mongo["tiktokbot"]["ryuzakilib"]
            return db[self.collection_name]



import asyncio
import json


class YoutubeJsonConvert:
    @staticmethod
    async def cookies_loads(file_json: str):
        with open(file_json, "r") as f:
            cookies = json.load(f)
        return cookies

    @staticmethod
    async def to_cookies(file_json: str, name_txt="cookies.txt"):
        with open(name_txt, 'w') as f:
            f.write("# Netscape HTTP Cookie File\n")
            f.write("# This is a generated file! Do not edit.\n\n")
            cookies = await YoutubeJsonConvert.cookies_loads(file_json)
            for cookie in cookies:
                f.write("\t".join([
                    cookie.get('domain', ''),
                    'TRUE' if cookie.get('hostOnly', False) else 'TRUE',
                    cookie.get('path', '/'),
                    'TRUE' if cookie.get('secure', False) else 'TRUE',
                    str(int(cookie.get('expirationDate', 0))),
                    cookie.get('name', ''),
                    cookie.get('value', ''),
                ]) + "\n")
        OUTPUT = "Successful output cookies.txt"
        return OUTPUT



import textwrap as tr
from typing import List, Optional

import matplotlib.pyplot as plt
import openai
import plotly.express as px
from openai.datalib.numpy_helper import numpy as np
from openai.datalib.pandas_helper import pandas as pd
from scipy import spatial
from sklearn.decomposition import PCA
from sklearn.manifold import TSNE
from sklearn.metrics import average_precision_score, precision_recall_curve
from tenacity import retry, stop_after_attempt, wait_random_exponential


@retry(wait=wait_random_exponential(min=1, max=20), stop=stop_after_attempt(6))
def get_embedding(text: str, engine="text-similarity-davinci-001", **kwargs) -> List[float]:
    # replace newlines, which can negatively affect performance.
    text = text.replace("\n", " ")

    return openai.Embedding.create(input=[text], engine=engine, **kwargs)["data"][0]["embedding"]


@retry(wait=wait_random_exponential(min=1, max=20), stop=stop_after_attempt(6))
async def aget_embedding(text: str, engine="text-similarity-davinci-001", **kwargs) -> List[float]:
    # replace newlines, which can negatively affect performance.
    text = text.replace("\n", " ")

    return (await openai.Embedding.acreate(input=[text], engine=engine, **kwargs))["data"][0][
        "embedding"
    ]


@retry(wait=wait_random_exponential(min=1, max=20), stop=stop_after_attempt(6))
def get_embeddings(
    list_of_text: List[str], engine="text-similarity-babbage-001", **kwargs
) -> List[List[float]]:
    assert len(list_of_text) <= 2048, "The batch size should not be larger than 2048."

    # replace newlines, which can negatively affect performance.
    list_of_text = [text.replace("\n", " ") for text in list_of_text]

    data = openai.Embedding.create(input=list_of_text, engine=engine, **kwargs).data
    return [d["embedding"] for d in data]


@retry(wait=wait_random_exponential(min=1, max=20), stop=stop_after_attempt(6))
async def aget_embeddings(
    list_of_text: List[str], engine="text-similarity-babbage-001", **kwargs
) -> List[List[float]]:
    assert len(list_of_text) <= 2048, "The batch size should not be larger than 2048."

    # replace newlines, which can negatively affect performance.
    list_of_text = [text.replace("\n", " ") for text in list_of_text]

    data = (await openai.Embedding.acreate(input=list_of_text, engine=engine, **kwargs)).data
    return [d["embedding"] for d in data]


def cosine_similarity(a, b):
    return np.dot(a, b) / (np.linalg.norm(a) * np.linalg.norm(b))


def plot_multiclass_precision_recall(y_score, y_true_untransformed, class_list, classifier_name):
    """
    Precision-Recall plotting for a multiclass problem. It plots average precision-recall, per class precision recall and reference f1 contours.

    Code slightly modified, but heavily based on https://scikit-learn.org/stable/auto_examples/model_selection/plot_precision_recall.html
    """
    n_classes = len(class_list)
    y_true = pd.concat(
        [(y_true_untransformed == class_list[i]) for i in range(n_classes)], axis=1
    ).values

    # For each class
    precision = dict()
    recall = dict()
    average_precision = dict()
    for i in range(n_classes):
        precision[i], recall[i], _ = precision_recall_curve(y_true[:, i], y_score[:, i])
        average_precision[i] = average_precision_score(y_true[:, i], y_score[:, i])

    # A "micro-average": quantifying score on all classes jointly
    precision_micro, recall_micro, _ = precision_recall_curve(y_true.ravel(), y_score.ravel())
    average_precision_micro = average_precision_score(y_true, y_score, average="micro")
    print(
        str(classifier_name)
        + " - Average precision score over all classes: {0:0.2f}".format(average_precision_micro)
    )

    # setup plot details
    plt.figure(figsize=(9, 10))
    f_scores = np.linspace(0.2, 0.8, num=4)
    lines = []
    labels = []
    for f_score in f_scores:
        x = np.linspace(0.01, 1)
        y = f_score * x / (2 * x - f_score)
        (l,) = plt.plot(x[y >= 0], y[y >= 0], color="gray", alpha=0.2)
        plt.annotate("f1={0:0.1f}".format(f_score), xy=(0.9, y[45] + 0.02))

    lines.append(l)
    labels.append("iso-f1 curves")
    (l,) = plt.plot(recall_micro, precision_micro, color="gold", lw=2)
    lines.append(l)
    labels.append("average Precision-recall (auprc = {0:0.2f})" "".format(average_precision_micro))

    for i in range(n_classes):
        (l,) = plt.plot(recall[i], precision[i], lw=2)
        lines.append(l)
        labels.append(
            "Precision-recall for class `{0}` (auprc = {1:0.2f})"
            "".format(class_list[i], average_precision[i])
        )

    fig = plt.gcf()
    fig.subplots_adjust(bottom=0.25)
    plt.xlim([0.0, 1.0])
    plt.ylim([0.0, 1.05])
    plt.xlabel("Recall")
    plt.ylabel("Precision")
    plt.title(f"{classifier_name}: Precision-Recall curve for each class")
    plt.legend(lines, labels)


def distances_from_embeddings(
    query_embedding: List[float],
    embeddings: List[List[float]],
    distance_metric="cosine",
) -> List[List]:
    """Return the distances between a query embedding and a list of embeddings."""
    distance_metrics = {
        "cosine": spatial.distance.cosine,
        "L1": spatial.distance.cityblock,
        "L2": spatial.distance.euclidean,
        "Linf": spatial.distance.chebyshev,
    }
    distances = [
        distance_metrics[distance_metric](query_embedding, embedding) for embedding in embeddings
    ]
    return distances


def indices_of_nearest_neighbors_from_distances(distances) -> np.ndarray:
    """Return a list of indices of nearest neighbors from a list of distances."""
    return np.argsort(distances)


def pca_components_from_embeddings(embeddings: List[List[float]], n_components=2) -> np.ndarray:
    """Return the PCA components of a list of embeddings."""
    pca = PCA(n_components=n_components)
    array_of_embeddings = np.array(embeddings)
    return pca.fit_transform(array_of_embeddings)


def tsne_components_from_embeddings(
    embeddings: List[List[float]], n_components=2, **kwargs
) -> np.ndarray:
    """Returns t-SNE components of a list of embeddings."""
    # use better defaults if not specified
    if "init" not in kwargs.keys():
        kwargs["init"] = "pca"
    if "learning_rate" not in kwargs.keys():
        kwargs["learning_rate"] = "auto"
    tsne = TSNE(n_components=n_components, **kwargs)
    array_of_embeddings = np.array(embeddings)
    return tsne.fit_transform(array_of_embeddings)


def chart_from_components(
    components: np.ndarray,
    labels: Optional[List[str]] = None,
    strings: Optional[List[str]] = None,
    x_title="Component 0",
    y_title="Component 1",
    mark_size=5,
    **kwargs,
):
    """Return an interactive 2D chart of embedding components."""
    empty_list = ["" for _ in components]
    data = pd.DataFrame(
        {
            x_title: components[:, 0],
            y_title: components[:, 1],
            "label": labels if labels else empty_list,
            "string": ["<br>".join(tr.wrap(string, width=30)) for string in strings]
            if strings
            else empty_list,
        }
    )
    chart = px.scatter(
        data,
        x=x_title,
        y=y_title,
        color="label" if labels else None,
        symbol="label" if labels else None,
        hover_data=["string"] if strings else None,
        **kwargs,
    ).update_traces(marker=dict(size=mark_size))
    return chart


def chart_from_components_3D(
    components: np.ndarray,
    labels: Optional[List[str]] = None,
    strings: Optional[List[str]] = None,
    x_title: str = "Component 0",
    y_title: str = "Component 1",
    z_title: str = "Compontent 2",
    mark_size: int = 5,
    **kwargs,
):
    """Return an interactive 3D chart of embedding components."""
    empty_list = ["" for _ in components]
    data = pd.DataFrame(
        {
            x_title: components[:, 0],
            y_title: components[:, 1],
            z_title: components[:, 2],
            "label": labels if labels else empty_list,
            "string": ["<br>".join(tr.wrap(string, width=30)) for string in strings]
            if strings
            else empty_list,
        }
    )
    chart = px.scatter_3d(
        data,
        x=x_title,
        y=y_title,
        z=z_title,
        color="label" if labels else None,
        symbol="label" if labels else None,
        hover_data=["string"] if strings else None,
        **kwargs,
    ).update_traces(marker=dict(size=mark_size))
    return chart



from sys import platform

from ..types.py_object import PyObject


class ScreenInfo(PyObject):
    def __init__(
        self,
        x: int,
        y: int,
        width: int,
        height: int,
        is_primary: bool,
        title: str,
    ):
        self.offset_x = x
        self.offset_y = y
        self.width = width
        self.height = height
        self.is_primary = is_primary
        self.title = title
        self.ffmpeg_parameters = ["-f"]

    def build_ffmpeg_command(self, frame_rate: int):
        if platform == "win32":
            path = "desktop"

            self.ffmpeg_parameters += [
                "gdigrab",
                "-offset_x",
                str(self.offset_x),
                "-offset_y",
                str(self.offset_y),
            ]
        else:
            path = f":0.0+{self.offset_x},{self.offset_y}"
            self.ffmpeg_parameters += ["x11grab"]

        self.ffmpeg_parameters += [
            "-video_size",
            f"{self.width}x{self.height}",
            "-framerate",
            str(frame_rate),
        ]
        return path



import asyncio
from typing import Callable, Dict, List


class HandlersHolder:
    def __init__(self):
        self._on_event_update: Dict[str, List[Callable]] = {
            "STREAM_END_HANDLER": [],
            "RAW_UPDATE_HANDLER": [],
            "INVITE_HANDLER": [],
            "KICK_HANDLER": [],
            "CLOSED_HANDLER": [],
            "LEFT_HANDLER": [],
            "PARTICIPANTS_LIST": [],
        }

    async def propagate(
        self,
        event_name: str,
        *args,
        **kwargs,
    ):
        for event in self._on_event_update[event_name]:
            asyncio.ensure_future(event(*args, **kwargs))

    def add_handler(
        self,
        event_name: str,
        func: Callable,
    ):
        self._on_event_update[event_name].append(func)



from typing import Tuple, Union

from pyrogram import Client, filters
from pyrogram.types import Message


class ChannelCopy:
    def __init__(self, client: Client, chat_id: Union[str, int] = None):
        self.client = client
        self.chat_id = chat_id

    async def download(
        self, message_id: int = None, faster: bool = False, to_forward: bool = False
    ) -> Union[Tuple[str, str], Message, str]:
        try:
            if faster:
                processed = await self.client.get_messages(
                    chat_id=self.chat_id, message_ids=message_id
                )
                photo_or_video = await processed.download()
                return [photo_or_video, processed.caption]
            elif to_forward:
                processed = await self.client.get_messages(
                    chat_id=self.chat_id, message_ids=message_id
                )
                return await processed.forward(self.chat_id)
            else:
                return "Please use required to_forward: boolean"
        except Exception as e:
            return f"Error: {e}"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from typing import Union

import requests


class API:
    def __init__(self, bot_token: str = None):
        self.bot_token = bot_token

    def telegram(self, method):
        url = f"https://api.telegram.org/bot{self.bot_token}/{method}"
        return url

    def get_me(self, re_json: bool = False):
        urls = self.telegram("getMe")
        headers = {
            "accept": "application/json",
            "User-Agent": "Telegram Bot SDK - (https://github.com/irazasyed/telegram-bot-sdk)",
        }
        if re_json:
            response = requests.post(urls, headers=headers).json()
            return response
        else:
            response = requests.post(urls, headers=headers)
            return response

    def forward_message(
        self,
        chat_id: Union[str, int] = None,
        from_chat_id: Union[str, int] = None,
        disable_notification: bool = False,
        message_id: int = None,
        re_json: bool = False,
    ):
        urls = self.telegram("forwardMessage")
        payload = {
            "chat_id": chat_id,
            "from_chat_id": from_chat_id,
            "message_id": message_id,
            "disable_notification": disable_notification,
        }
        headers = {
            "accept": "application/json",
            "User-Agent": "Telegram Bot SDK - (https://github.com/irazasyed/telegram-bot-sdk)",
            "content-type": "application/json",
        }
        if re_json:
            response = requests.post(urls, json=payload, headers=headers).json()
            return response
        else:
            response = requests.post(urls, json=payload, headers=headers)
            return response

    def send_photo(
        self,
        chat_id: Union[str, int] = None,
        photo: Union[str, None] = None,
        caption: str = None,
        disable_notification: bool = False,
        reply_to_message_id: int = None,
    ):
        urls = self.telegram("sendPhoto")
        payload = {
            "chat_id": chat_id,
            "photo": photo,
            "caption": caption,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
        }
        headers = {
            "accept": "application/json",
            "User-Agent": "Telegram Bot SDK - (https://github.com/irazasyed/telegram-bot-sdk)",
            "content-type": "application/json",
        }
        if re_json:
            response = requests.post(urls, json=payload, headers=headers).json()
            return response
        else:
            response = requests.post(urls, json=payload, headers=headers)
            return response

    def send_audio(
        self,
        chat_id: Union[str, int] = None,
        audio: Union[str, None] = None,
        duration: int = None,
        performer=None,
        track=None,
        disable_notification: bool = False,
        reply_to_message_id: int = None,
        re_json: bool = False,
    ):
        urls = self.telegram("sendAudio")
        payload = {
            "chat_id": chat_id,
            "audio": audio,
            "duration": duration,
            "performer": performer,
            "track": track,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
        }
        headers = {
            "accept": "application/json",
            "User-Agent": "Telegram Bot SDK - (https://github.com/irazasyed/telegram-bot-sdk)",
            "content-type": "application/json",
        }
        if re_json:
            response = requests.post(urls, json=payload, headers=headers)
            return response
        else:
            response = requests.post(urls, json=payload, headers=headers)
            return response

    def send_document(
        self,
        chat_id: Union[str, int] = None,
        document: Union[str, None] = None,
        caption: str = None,
        disable_notification: bool = False,
        reply_to_message_id: int = None,
        re_json: bool = False,
    ):
        urls = self.telegram("sendDocument")
        payload = {
            "chat_id": chat_id,
            "document": document,
            "caption": caption,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
        }
        headers = {
            "accept": "application/json",
            "User-Agent": "Telegram Bot SDK - (https://github.com/irazasyed/telegram-bot-sdk)",
            "content-type": "application/json",
        }
        if re_json:
            response = requests.post(urls, json=payload, headers=headers)
            return response
        else:
            response = requests.post(urls, json=payload, headers=headers)
            return response

    def send_sticker(
        self,
        chat_id: Union[str, int] = None,
        sticker: Union[str, None] = None,
        caption: str = None,
        disable_notification: bool = False,
        reply_to_message_id: int = None,
        re_json: bool = False,
    ):
        urls = self.telegram("sendSticker")
        payload = {
            "chat_id": chat_id,
            "sticker": sticker,
            "caption": caption,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
        }
        headers = {
            "accept": "application/json",
            "User-Agent": "Telegram Bot SDK - (https://github.com/irazasyed/telegram-bot-sdk)",
            "content-type": "application/json",
        }
        if re_json:
            response = requests.post(urls, json=payload, headers=headers)
            return response
        else:
            response = requests.post(urls, json=payload, headers=headers)
            return response

    def send_message(
        self,
        chat_id: Union[str, int] = None,
        text: str = None,
        disable_web_page_preview: bool = False,
        disable_notification: bool = False,
        reply_to_message_id: int = None,
        re_json: bool = False,
    ):
        urls = self.telegram("sendMessage")
        payload = {
            "text": text,
            "parse_mode": parse_mode,
            "disable_web_page_preview": disable_web_page_preview,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_id": chat_id,
        }
        headers = {
            "accept": "application/json",
            "User-Agent": "Telegram Bot SDK - (https://github.com/irazasyed/telegram-bot-sdk)",
            "content-type": "application/json",
        }
        if re_json:
            response = requests.post(urls, json=payload, headers=headers).json()
            return response
        else:
            response = requests.post(urls, json=payload, headers=headers)
            return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import pathlib
from typing import Optional, Union

import google.generativeai as genai
import requests
from pymongo import MongoClient


class GeminiLatest:
    def __init__(
        self,
        api_keys: Optional[str] = None,
        mongo_url: Optional[str] = None,
        model: Optional[str] = "gemini-1.5-flash",
        user_id: Optional[int] = None,
        generation_configs = {}
    ):
        self.api_keys = api_keys
        self.model = model
        self.generation_configs = generation_configs
        self.user_id = user_id
        self.mongo_url = mongo_url
        self.client = MongoClient(self.mongo_url)
        self.db = self.client.tiktokbot
        self.collection = self.db.users

        genai.configure(api_key=self.api_keys)

    def _close(self):
        self.client.close()

    def _get_gemini_chat_from_db(self):
        user_data = self.collection.find_one({"user_id": self.user_id})
        return user_data.get("gemini_chat", []) if user_data else []

    def _update_gemini_chat_in_db(self, gemini_chat):
        self.collection.update_one(
            {"user_id": self.user_id},
            {"$set": {"gemini_chat": gemini_chat}},
            upsert=True
        )

    def _clear_history_in_db(self):
        unset_clear = {"gemini_chat": None}
        return self.collection.update_one({"user_id": self.user_id}, {"$unset": unset_clear})

    def clear_database(self):
        """Clear the gemini_chat history for the current user."""
        result = self._clear_history_in_db()
        if result.modified_count > 0:
            return "Chat history cleared successfully."
        else:
            return "No chat history found to clear."

    def geni_upload_file(self, file_image, mine_type="image/jpeg"):
        cookie_picture = {
            "mime_type": mine_type,
            "data": pathlib.Path(file_image).read_bytes()
        }
        return cookie_picture

    def get_response_image(self, query, file_image):
        model_image = genai.GenerativeModel("gemini-1.5-flash")
        cookie_picture = self.geni_upload_file(file_image)
        response = model_image.generate_content(
            contents=[query, cookie_picture]
        )
        return response.text

    def __get_response_gemini(self, query: str = None, settings_config: bool = False):
        try:
            if settings_config:
                generation_config = self.generation_configs
            else:
                generation_config = {
                    "temperature": 1,
                    "top_p": 0.95,
                    "top_k": 64,
                    "max_output_tokens": 8192,
                    "response_mime_type": "text/plain",
                }
            model_flash = genai.GenerativeModel(
                model_name=self.model,
                generation_config=generation_config,
                safety_settings={
                    genai.types.HarmCategory.HARM_CATEGORY_HATE_SPEECH: genai.types.HarmBlockThreshold.BLOCK_NONE,
                    genai.types.HarmCategory.HARM_CATEGORY_HARASSMENT: genai.types.HarmBlockThreshold.BLOCK_NONE,
                    genai.types.HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT: genai.types.HarmBlockThreshold.BLOCK_NONE,
                    genai.types.HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT: genai.types.HarmBlockThreshold.BLOCK_NONE,
                }
            )
            gemini_chat = self._get_gemini_chat_from_db()
            gemini_chat.append({"role": "user", "parts": [{"text": query}]})
            chat_session = model_flash.start_chat(history=gemini_chat)
            response_data = chat_session.send_message(query)
            answer = response_data.text
            gemini_chat.append({"role": "model", "parts": [{"text": answer}]})
            self._update_gemini_chat_in_db(gemini_chat)
            return answer, gemini_chat
        except Exception as e:
            error_msg = f"Rate limit exceeded after multiple attempts."
            return error_msg, gemini_chat



import requests

from RyuzakiLib.api.reqs import async_search


class AkenoAI:
    def __init__(self, base_api_dev: str = "https://akeno.randydev.my.id"):
        self.base_api_dev = base_api_dev
        self.connected = False
        self.api_key = None

    async def signup(self, gmail: str, username: str):
        if not gmail.endswith("@gmail.com"):
            return "Invalid gmail"
        params = {"username": username, "gmail": gmail}
        response = requests.post(
            self.base_api_dev + "/register",
            params=params
        ).json()
        return response

    async def connect(self, username: str, requests_limit: int = 30):
        response = await async_search(
            f"{self.base_api_dev}/get_api_key?api_key={username}",
            re_json=True
        )
        if response.get("requests_made", 0) >= requests_limit:
            return "The limit has been reached"
        else:
            self.connected = True
            self.api_key = username
            return None

    async def hentai(self, query: str):
        if not self.connected or not self.api_key:
            return "Not connected or API key missing"
        response = await async_search(
            f"{self.base_api_dev}/akeno/hentai?query={query}&api_key={self.api_key}",
            re_json=True
        )
        return response

    async def pornopics(self, query: str):
        if not self.connected or not self.api_key:
            return "Not connected or API key missing"
        response = await async_search(
            f"{self.base_api_dev}/akeno/pornpics?query={query}&api_key={self.api_key}",
            re_json=True
        )
        return response

    async def x_search(self, query: str):
        if not self.connected or not self.api_key:
            return "Not connected or API key missing"
        url = f"{self.base_api_dev}/akeno/xnxxsearch-v2?query={query}&api_key={self.api_key}"
        response = await async_search(url, re_json=True)
        return response

    async def x_download(self, url: str):
        if not self.connected or not self.api_key:
            return "Not connected or API key missing"
        url_ = f"{self.base_api_dev}/akeno/xnxx-dl-v2?link={url}&api_key={self.api_key}"
        response = await async_search(url_, re_json=True)
        return response

    async def delete_api_key(self, email: str):
        response = requests.delete(
            f"{self.base_api_dev}/delete_api_key_by_username?email={email}"
        ).json()
        return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2020-2023 (c) Randy W @xtdevs, @xtsea
#
# from : https://github.com/TeamKillerX
# Channel : @RendyProjects
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import base64
from base64 import b64decode
from io import BytesIO
from typing import Optional

import requests
from aiohttp import ClientSession
from gpytranslate import SyncTranslator
from pyrogram import Client, filters
from pyrogram.types import Message

aiosession = ClientSession()


class Carbon:
    @staticmethod
    async def make_carbon(
        code: str = None,
        title: Optional[str] = "Ryuzaki",
        theme: Optional[str] = "breeze",
        auto_translate: Optional[str] = "auto",
        darkmode: Optional[bool] = True,
        rayso=False
    ):
        base="aHR0cHM6Ly9hcGkuc2Fmb25lLmRldi9yYXlzbw=="
        filename = "rayso.jpg"
        if rayso:
            api_url = b64decode(base).decode("utf-8")
            x = requests.post(
                f"{api_url}",
                json={
                    "code": code,
                    "title": title,
                    "theme": theme,
                    "padding": 64,
                    "language": auto_translate,
                    "darkMode": darkmode,
                },
            )
            if x.status_code != 200:
                return "Error api Gay"
            data = x.json()
            try:
                image_data = base64.b64decode(data["image"])
                f = BytesIO(image_data)
                f.name = filename
                return f
            except Exception as e:
                return f"Error: {e}"
        else:
            url = "https://carbonara.solopov.dev/api/cook"
            async with aiosession.post(url, json={"code": code}) as resp:
                image = BytesIO(await resp.read())
            image.name = "carbon.png"
            return image



from typing import Union

from pyrogram import Client, filters
from pyrogram.types import Message


class RyuzakiPowers:
    def __init__(self):
        pass

    def ryuzaki_ban(self, blacklist: Union[int, list] = None, func=None):
        def wrapper(client: Client, message: Message):
            user_id = message.from_user.id if message.from_user else None
            if user_id in blacklist:
                return f"User {user_id} is banned."
            else:
                return func(client, message)

        return wrapper

    def ryuzaki_owner(self, user: Union[int, list] = None, sudo: bool = None, func=None):
        def wrapper(client: Client, message: Message):
            if sudo:
                user_id = message.from_user.id if message.from_user else None
                if user_id != user:
                    return func(client, message)
                else:
                    return "Only Developed"
            else:
                return "Something error"

        return wrapper



import requests

from RyuzakiLib.api.reqs import AsyicXSearcher


class FluxAi:
    @staticmethod
    async def schellwithflux(args, auto_enhancer=False, re_content=False):
        API_URL = "https://randydev-ryuzaki-api.hf.space/api/v1/akeno/fluxai"
        payload = {
            "user_id": 1191668125,  # Please don't edit here
            "args": args,
            "auto_enhancer": auto_enhancer
        }
        response = requests.post(API_URL, json=payload)
        return response.content if re_content else response.json()



import requests


class CloneBot:
    def __init__(self, api_key: str = None, bot_token: str = None):
        self.api_key = api_key
        self.bot_token = bot_token

    def _clone_chatgpt(self, clone=False, re_json=False):
        url = f"https://randydev.my.id/clone/chatgpt-bot?bot_token={self.bot_token}"
        headers = {"accept": "application/json", "api-key": self.api_key}
        if clone:
            if re_json:
                try:
                    response = requests.post(url, headers=headers).json()
                    return response
                except Exception as e:
                    return f"Error request: {e}"
            else:
                try:
                    response = requests.post(url, headers=headers)
                    return response.text
                except Exception as e:
                    return f"Error request: {e}"
        else:
            return "Error required clone=True"
